@@ -10,12 +10,12 @@ use autodie;
 use Carp;
 use Scalar::Util qw/openhandle/;
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 #@EXPORT_OK = qw//;
 
 BEGIN {
-  my @accessors = qw/fh file line headers/;
+  my @accessors = qw/fh file line headers fields/;
 
   for my $accessor ( @accessors ) {
     no strict 'refs';
@@ -66,6 +66,10 @@ sub new {
     $self->{names} = [ $self->readheader() ];
   }
 
+  $self->{fields} = $self->{names};
+
+  $self->{empty_as_undef} //= 0;
+
   return $self;
 }
 
@@ -83,13 +87,15 @@ sub readheader {
     my @fields = split /\t/,$line;
 
     unless ( $line =~ /^#/ ) {
-      croak("Did not find fields header: $line");
+      croak("Did not find required fields and types header lines: $line");
     }
 
-    if ( "#fields" eq shift(@fields) ) {
+    my $type = shift(@fields);
+    if ( "#fields" eq  $type ) {
       # yay.
       # we have our field names...
       @names = @fields;
+    } elsif ( "#types" eq $type) {
       last;
     }
   }
@@ -131,10 +137,13 @@ sub getLine {
     }
 
     for my $name ( @names ) {
-      if ( ( $fields[0] eq "-" ) || $fields[0] eq "(empty)" ) {
-        shift(@fields); # empty field
+      my $field = shift(@fields);
+      if ( ( $field eq "-" ) ) {
+        $f{$name} = undef;
+      } elsif ( $field eq "(empty)" ) {
+        $f{$name} = $self->{empty_as_undef} ? undef : [];
       } else {
-        $f{$name} = shift(@fields);
+        $f{$name} = $field;
       }
     }
 
